﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Reflection;

namespace howto_list_ancestors
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnFind_Click(object sender, EventArgs e)
        {
            lblNumClasses.Text = "";
            lstClasses.Items.Clear();
            lstAncestors.Items.Clear();
            Cursor = Cursors.WaitCursor;
            Refresh();

            // List the loaded assemblies.
            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
                Console.WriteLine(assembly.GetName().Name);
            int num_assemblies = AppDomain.CurrentDomain.GetAssemblies().Length;
            Console.WriteLine(num_assemblies.ToString() + " assemblies");

            // Get the type entered.
            string type_name = txtType.Text.ToLower();
            var type_query =
                from assembly in AppDomain.CurrentDomain.GetAssemblies()
                from type in assembly.GetTypes()
                where (type.Name.ToLower() == type_name)
                select type;
            Type target_type = type_query.FirstOrDefault();

            if (target_type != null)
            {
                // Get classes that are assignable to the target type.
                var classes =
                    from assembly in AppDomain.CurrentDomain.GetAssemblies()
                    from type in assembly.GetTypes()
                    where target_type.IsAssignableFrom(type)
                    select type;

                // Display the classes.
                foreach (Type type in classes)
                {
                    lstClasses.Items.Add(new TypeInfo(type));
                }
            }

            // Display the number of classes.
            lblNumClasses.Text = lstClasses.Items.Count.ToString() + " classes";
            Cursor = Cursors.Default;
        }

        // Display the ancestors of the selected class.
        private void lstClasses_SelectedIndexChanged(object sender, EventArgs e)
        {
            lstAncestors.Items.Clear();
            if (lstClasses.SelectedIndex == -1) return;
            Cursor = Cursors.WaitCursor;
            Refresh();

            // Get the type.
            List<TypeInfo> types = new List<TypeInfo>();
            TypeInfo the_type = (TypeInfo)lstClasses.SelectedItem;
            types.Add(the_type);
            for (Type parent = the_type.TheType.BaseType;
                parent != null;
                parent = parent.BaseType)
            {
                types.Insert(0, new TypeInfo(parent));
            }
            for (int i = 0; i < types.Count; i++)
            {
                lstAncestors.Items.Add(new string(' ', i * 2) + types[i].TheType.Name);
            }

            Cursor = Cursors.Default;
        }
    }
}
